from dataclasses import dataclass


@dataclass(unsafe_hash=True, frozen=True, order=True)
class Variable:
    name: str


@dataclass(unsafe_hash=True, order=True)
class Literal:
    var: Variable
    a: float


@dataclass(unsafe_hash=True, frozen=True)
class Equation:
    literals: list[Literal]
    b: float


@dataclass(unsafe_hash=True, order=True)
class Substitution:
    var: Variable
    value: float


@dataclass(unsafe_hash=True)
class Function:
    literals: list[Literal]
    tends: int

    def compute(self, substitutions: list[Substitution]) -> float:
        result = 0

        for literal, substitution in zip(sorted(self.literals), sorted(substitutions)):
            result += literal.a * substitution.value

        return result

    def compute_with_tends(self, substitutions: list[Substitution]) -> float:
        return -self.compute(substitutions) if self.tends == 1 else self.compute(substitutions)